@@ -3,7 +3,7 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-# Enable 'with' statements in Python 2.5
+from __future__ import print_function
 from __future__ import with_statement
 
 import os.path
@@ -15,11 +15,11 @@ import sys
 import time
 
 from buildbot_lib import (
-    BuildContext, BuildStatus, Command, EnsureDirectoryExists,
+    BuildContext, BuildStatus, Command, EnsureDirectoryExists, GNArch,
     ParseStandardCommandLine, RemoveDirectory, RemovePath,
     RemoveGypBuildDirectories, RemoveSconsBuildDirectories, RunBuild, SCons,
-    SetupLinuxEnvironment, SetupMacEnvironment, SetupWindowsEnvironment,
-    SetupAndroidEnvironment, Step, StepLink, StepText, TryToCleanContents,
+    SetupLinuxEnvironment, SetupWindowsEnvironment,
+    Step, StepLink, StepText, TryToCleanContents,
     RunningOnBuildbot)
 
 
@@ -75,84 +75,163 @@ def ArchiveCoverage(context):
       'cp', '-R', '-a', 'public-read',
       'html', gs_path,
   ], cwd=cov_dir)
-  print '@@@STEP_LINK@view@%s@@@' % link_url
+  print('@@@STEP_LINK@view@%s@@@' % link_url)
 
 
-def CommandGypBuild(context):
-  # Do not use goma when inside a toolchain build, because the
-  # freshly-built NaCl compilers will never be available via goma.
-  # This sacrifices the benefits of goma for building the trusted
-  # code too, but it's not clear how to teach Gyp to use goma for
-  # some compilers and not others.
-  use_goma = (RunningOnBuildbot() and
-              not context['no_goma'] and
-              not context['inside_toolchain'])
+def DoGNBuild(status, context, force_clang=False, force_arch=None):
+  if context['no_gn']:
+    return False
 
-  if use_goma:
-    # Since this is for buildbot, it should not be good to use the result
-    # generated by the different version compiler.
-    os.environ['GOMA_HERMETIC'] = 'fallback'
+  use_clang = force_clang or context['clang']
+
+  # Linux builds (or cross-builds) for every target.  Mac builds for
+  # x86-32 and x86-64, and can build untrusted code for others.
+  if context.Windows() and context['arch'] != '64':
+    # The GN scripts for MSVC barf for a target_cpu other than x86 or x64
+    # even if we only try to build the untrusted code.  Windows does build
+    # for both x86-32 and x86-64 targets, but the GN Windows MSVC toolchain
+    # scripts only support x86-64 hosts--and the Windows build of Clang
+    # only has x86-64 binaries--while NaCl's x86-32 testing bots have to be
+    # actual x86-32 hosts.
+    return False
+
+  if force_arch is not None:
+    arch = force_arch
   else:
-    os.environ['GOMA_DISABLED'] = '1'
+    arch = context['arch']
 
-  runtest_py = os.environ.get('RUNTEST')
-  alt_runtest_py = '/b/build/scripts/slave/runtest.py'
-  if runtest_py is None and os.path.exists(alt_runtest_py):
-    runtest_py = alt_runtest_py
+  if context.Linux():
+    # The Linux build uses a sysroot.  'gclient runhooks' installs this
+    # for the default architecture, but this might be a cross-build that
+    # gclient didn't know was going to be done.  The script completes
+    # quickly when it's redundant with a previous run.
+    with Step('update_sysroot', status):
+      sysroot_arch = {'arm': 'arm',
+                      '32': 'i386',
+                      '64': 'amd64',
+                      'mips32': 'mips'}[arch]
+      Command(context, cmd=[sys.executable,
+                            '../build/linux/sysroot_scripts/install-sysroot.py',
+                            '--arch=' + sysroot_arch])
 
-  # TODO(bradnelson): Figure out why win64 trybots can't upload goma logs.
-  buildername = os.environ.get('BUILDBOT_BUILDERNAME', '')
-  excluded_os = False
-  for name in ['win64', 'vista', 'win7-64', 'win8-64']:
-    if name in buildername:
-      excluded_os = True
+  out_suffix = '_' + arch
+  if force_clang:
+    out_suffix += '_clang'
+  gn_out = '../out' + out_suffix
 
-  if runtest_py is None or excluded_os:
-    # Fallback to direct goma + ninja if not run on bots.
-    try:
-      if use_goma:
-        Command(context, cmd=[
-            sys.executable, '/b/build/goma/goma_ctl.py', 'restart'])
-      cmd = ['ninja', '-v', '-k', '0', '-C', '../out/' + context['gyp_mode']]
-      if use_goma:
-        cmd += ['-j50']
-      Command(context, cmd=cmd)
-    finally:
-      if use_goma:
-        Command(context, cmd=[
-            sys.executable, '/b/build/goma/goma_ctl.py', 'stop'])
-  else:
-    # Infer the location of compile.py from runtest.py.
-    compile_py = os.path.join(os.path.dirname(runtest_py), 'compile.py')
-    cmd = [sys.executable, compile_py, '--target', context['gyp_mode'],
-           '--src-dir', '../', '--build-tool', 'ninja',
-           '--ninja-ensure-up-to-date']
-    if use_goma:
-      cmd += ['--compiler', 'goma']
-      cmd += ['--goma-dir', '/b/build/goma']
-    # Verbose and but stop on fail.
-    cmd += ['--', '-v', '-k', '0']
-    Command(context, cmd=cmd)
+  def BoolFlag(cond):
+    return 'true' if cond else 'false'
 
+  gn_newlib = BoolFlag(not context['use_glibc'])
+  gn_glibc = BoolFlag(context['use_glibc'])
+  gn_arch_name = GNArch(arch)
 
-def CommandGypGenerate(context):
-  Command(
-          context,
-          cmd=[sys.executable, 'native_client/build/gyp_nacl'],
-          cwd='..')
+  gn_gen_args = [
+      # The Chromium GN definitions might default enable_nacl to false
+      # in some circumstances, but various BUILD.gn files involved in
+      # the standalone NaCl build assume enable_nacl==true.
+      'enable_nacl=true',
+      'target_cpu="%s"' % gn_arch_name,
+      'is_debug=' + context['gn_is_debug'],
+      'use_gcc_glibc=' + gn_glibc,
+      'use_clang_newlib=' + gn_newlib,
+  ]
 
+  # is_clang is the GN default for Mac and Linux, so
+  # don't override that on "non-clang" bots, but do set
+  # it explicitly for an explicitly "clang" bot.
+  if use_clang:
+    gn_gen_args.append('is_clang=true')
 
-def CommandGclientRunhooks(context):
+  # The ASan runtime requires libstdc++, and the version on the bots is older
+  # than the version in the sysroot, so ASan-built sel_ldr from the sysroot
+  # won't run. For now we disable the sysroot (this matches the SCons build)
+  # but the term fix would be to just use GN's libcxx build.
+  if context['asan']:
+    gn_gen_args.append('use_sysroot=false')
+
+  # If this is a 32-bit build but the kernel reports as 64-bit,
+  # then gn will set host_cpu=x64 when we want host_cpu=x86.
+  if context.Linux() and arch == '32':
+    gn_gen_args.append('host_cpu="x86"')
+
+  # Mac can build the untrusted code for machines Mac doesn't
+  # support, but the GN files will get confused in a couple of ways.
+  if context.Mac() and arch not in ('32', '64'):
+    gn_gen_args += [
+        # Subtle GN issues mean that $host_toolchain context will
+        # wind up seeing current_cpu=="arm" when target_os is left
+        # to default to "mac", because host_toolchain matches
+        # _default_toolchain and toolchain_args() does not apply to
+        # the default toolchain.  Using target_os="ios" ensures that
+        # the default toolchain is something different from
+        # host_toolchain and thus that toolchain's toolchain_args()
+        # block will be applied and set current_cpu correctly.
+        'target_os="ios"',
+        # build/config/ios/ios_sdk.gni will try to find some
+        # XCode magic that won't exist.  This GN flag disables
+        # the problematic code.
+        'ios_enable_code_signing=false',
+        ]
+
+  gn_out_trusted = gn_out
+  gn_out_irt = os.path.join(gn_out, 'irt_' + gn_arch_name)
+
+  gn_cmd = [
+      'gn.bat' if context.Windows() else 'gn',
+      '--dotfile=../native_client/.gn', '--root=..',
+      # Note: quotes are not needed around this space-separated
+      # list of args.  The shell would remove them before passing
+      # them to a program, and Python bypasses the shell.  Adding
+      # quotes will cause an error because GN will see unexpected
+      # double quotes.
+      '--args=%s' % ' '.join(gn_gen_args),
+      'gen', gn_out,
+  ]
+
+  gn_ninja_cmd = ['ninja', '-C', gn_out, '-v']
+  if gn_arch_name not in ('x86', 'x64') and not context.Linux():
+    # On non-Linux non-x86, we can only build the untrusted code.
+    gn_ninja_cmd.append('untrusted')
+
+  with Step('gn_compile' + out_suffix, status):
+    Command(context, cmd=gn_cmd)
+    Command(context, cmd=gn_ninja_cmd)
+
+  return (gn_out_trusted, gn_out_irt)
+
+def DoGNTest(status, context, using_gn, gn_perf_prefix, gn_step_suffix):
+  if not using_gn:
+    return
+
+  gn_out_trusted, gn_out_irt = using_gn
+
+  # Non-Linux can build non-x86 untrusted code, but can't build or run
+  # the trusted code and so cannot test.
+  if context['arch'] not in ('32', '64') and not context.Linux():
+    return
+
+  gn_sel_ldr = os.path.join(gn_out_trusted, 'sel_ldr')
   if context.Windows():
-    gclient = 'gclient.bat'
-  else:
-    gclient = 'gclient'
-  print 'Running gclient runhooks...'
-  print 'GYP_CROSSCOMPILE=' + context.GetEnv('GYP_CROSSCOMPILE', '')
-  print 'GYP_GENERATORS=' + context.GetEnv('GYP_GENERATORS', '')
-  print 'GYP_MSVS_VERSION=' + context.GetEnv('GYP_MSVS_VERSION', '')
-  print 'GYP_DEFINES=' + context.GetEnv('GYP_DEFINES', '')
-  Command(context, cmd=[gclient, 'runhooks', '--force'])
+    gn_sel_ldr += '.exe'
+  gn_extra = [
+      'force_sel_ldr=' + gn_sel_ldr,
+      'force_irt=' + os.path.join(gn_out_irt, 'irt_core.nexe'),
+      'perf_prefix=' + gn_perf_prefix,
+      ]
+  if context.Linux():
+    gn_extra.append('force_bootstrap=' +
+                    os.path.join(gn_out_trusted, 'nacl_helper_bootstrap'))
+  def RunGNTests(step_suffix, extra_scons_modes, suite_suffix):
+    for suite in ['small_tests', 'medium_tests', 'large_tests']:
+      with Step(suite + step_suffix + gn_step_suffix, status,
+                halt_on_fail=False):
+        SCons(context,
+              mode=context['default_scons_mode'] + extra_scons_modes,
+              args=[suite + suite_suffix] + gn_extra)
+  if not context['use_glibc']:
+    RunGNTests('', [], '')
+  RunGNTests(' under IRT', ['nacl_irt_test'], '_irt')
 
 
 def BuildScript(status, context):
@@ -173,9 +252,9 @@ def BuildScript(status, context):
       context.SetEnv('TMP', tmp_dir)
     else:
       tmp_dir = '/tmp'
-    print 'Making sure %s exists...' % tmp_dir
+    print('Making sure %s exists...' % tmp_dir)
     EnsureDirectoryExists(tmp_dir)
-    print 'Cleaning up the contents of %s...' % tmp_dir
+    print('Cleaning up the contents of %s...' % tmp_dir)
     # Only delete files and directories like:
     #   */nacl_tmp/*
     # TODO(bradnelson): Drop this after a bit.
@@ -215,17 +294,17 @@ def BuildScript(status, context):
     if os.path.exists(nacl_tmp):
       for bot in os.listdir(nacl_tmp):
         bot_path = os.path.join(nacl_tmp, bot)
-        print 'Cleaning prior build temp dir: %s' % bot_path
+        print('Cleaning prior build temp dir: %s' % bot_path)
         sys.stdout.flush()
         if os.path.isdir(bot_path):
           for d in os.listdir(bot_path):
             path = os.path.join(bot_path, d)
-            print 'Removing leftover: %s' % path
+            print('Removing leftover: %s' % path)
             sys.stdout.flush()
             RemovePath(path)
           os.rmdir(bot_path)
         else:
-          print 'Removing rogue file: %s' % bot_path
+          print('Removing rogue file: %s' % bot_path)
           RemovePath(bot_path)
       os.rmdir(nacl_tmp)
     # Clean /tmp so we get a list of what's accumulating.
@@ -246,106 +325,29 @@ def BuildScript(status, context):
           "find /var/folders -name '.com.google.Chrome*' -exec rm -rfv '{}' ';'",
           shell=True)
 
-  # Skip over hooks when run inside the toolchain build because
-  # package_version would overwrite the toolchain build.
-  if inside_toolchain:
-    with Step('gyp_generate_only', status):
-      CommandGypGenerate(context)
-  else:
-    with Step('gclient_runhooks', status):
-      CommandGclientRunhooks(context)
+  # Always update Clang.  On Linux and Mac, it's the default for the GN build.
+  # It's also used for the Linux Breakpad build. On Windows, we do a second
+  # Clang GN build.
+  with Step('update_clang', status):
+    Command(context, cmd=[sys.executable, '../tools/clang/scripts/update.py'])
 
   # Make sure our GN build is working.
-  if context.Windows() and not context['clang']:
-    # The GN scripts for MSVC barf for a target_cpu other than x86 or
-    # x64 even if we only try to build the untrusted code.  Windows does
-    # build for both x86-32 and x86-64 targets, but the GN Windows MSVC
-    # toolchain scripts only support x86-64 hosts, while NaCl's x86-32
-    # testing bots have to be actual x86-32 hosts.
-    can_use_gn = context['arch'] == '64'
-  else:
-    # Linux builds (or cross-builds) for every target.  Mac builds for
-    # x86-32 and x86-64, and can build untrusted code for others.
-    can_use_gn = True
-  gn_out = '../out'
+  using_gn = DoGNBuild(status, context)
+  using_gn_clang = False
+  if (context.Windows() and
+      not context['clang'] and
+      context['arch'] in ('32', '64')):
+    # On Windows, do a second GN build with is_clang=true.
+    using_gn_clang = DoGNBuild(status, context, True)
 
-  if can_use_gn:
-    def BoolFlag(cond):
-      return 'true' if cond else 'false'
-
-    gn_newlib = BoolFlag(not context['use_glibc'])
-    gn_glibc = BoolFlag(context['use_glibc'])
-
-    gn_arch_name = {
-        'arm': 'arm',
-        '32': 'x86',
-        '64': 'x64'
-        }[context['arch']]
-
-    gn_gen_args = [
-      'target_cpu="%s"' % gn_arch_name,
-      'is_debug=' + context['gn_is_debug'],
-      'use_gcc_glibc=' + gn_glibc,
-      'use_clang_newlib=' + gn_newlib,
-    ]
-
-    # is_clang is the GN default for Mac and Windows, so
-    # don't override that on "non-clang" bots, but do set
-    # it explicitly for an explicitly "clang" bot.
-    if context['clang']:
-      gn_gen_args.append('is_clang=true')
-
-    # If this is a 32-bit build but the kernel reports as 64-bit,
-    # then gn will set host_cpu=x64 when we want host_cpu=x86.
-    if context.Linux() and context['arch'] == '32':
-      gn_gen_args.append('host_cpu="x86"')
-
-    # Mac can build the untrusted code for machines Mac doesn't
-    # support, but the GN files will get confused in a couple of ways.
-    if context.Mac() and context['arch'] not in ('32', '64'):
-      gn_gen_args += [
-          # Subtle GN issues mean that $host_toolchain context will
-          # wind up seeing current_cpu=="arm" when target_os is left
-          # to default to "mac", because host_toolchain matches
-          # _default_toolchain and toolchain_args() does not apply to
-          # the default toolchain.  Using target_os="ios" ensures that
-          # the default toolchain is something different from
-          # host_toolchain and thus that toolchain's toolchain_args()
-          # block will be applied and set current_cpu correctly.
-          'target_os="ios"',
-          # build/config/ios/ios_sdk.gni will try to find some
-          # XCode magic that won't exist.  This GN flag disables
-          # the problematic code.
-          'ios_enable_code_signing=false',
-          ]
-
-    gn_out_trusted = gn_out
-    gn_out_irt = os.path.join(gn_out, 'irt_' + gn_arch_name)
-
-    gn_cmd = [
-      'gn.bat' if context.Windows() else 'gn',
-      '--dotfile=../native_client/.gn', '--root=..',
-      # Note: quotes are not needed around this space-separated
-      # list of args.  The shell would remove them before passing
-      # them to a program, and Python bypasses the shell.  Adding
-      # quotes will cause an error because GN will see unexpected
-      # double quotes.
-      '--args=%s' % ' '.join(gn_gen_args),
-      'gen', gn_out,
-      ]
-
-    gn_ninja_cmd = ['ninja', '-C', gn_out]
-    if gn_arch_name not in ('x86', 'x64') and not context.Linux():
-      # On non-Linux non-x86, we can only build the untrusted code.
-      gn_ninja_cmd.append('untrusted')
-
-    with Step('gn_compile', status):
-      Command(context, cmd=gn_cmd)
-      Command(context, cmd=gn_ninja_cmd)
-
-  if context['clang']:
-    with Step('update_clang', status):
-      Command(context, cmd=[sys.executable, '../tools/clang/scripts/update.py'])
+  if context.Windows() and context['arch'] == '64':
+    # On Windows, do a second pair of GN builds for 32-bit.  The 32-bit
+    # bots can't do GN builds at all, because the toolchains GN uses don't
+    # support 32-bit hosts.  The 32-bit binaries built here cannot be
+    # tested on a 64-bit host, but compile time issues can be caught.
+    DoGNBuild(status, context, False, '32')
+    if not context['clang']:
+      DoGNBuild(status, context, True, '32')
 
   # Just build both bitages of validator and test for --validator mode.
   if context['validator']:
@@ -381,18 +383,39 @@ def BuildScript(status, context):
   with Step('checkdeps', status):
     Command(context, cmd=[sys.executable, 'tools/checkdeps/checkdeps.py'])
 
-  # Make sure our Gyp build is working.
-  if not context['no_gyp']:
-    with Step('gyp_compile', status):
-      CommandGypBuild(context)
-
   # On a subset of Linux builds, build Breakpad tools for testing.
   if context['use_breakpad_tools']:
     with Step('breakpad configure', status):
       Command(context, cmd=['mkdir', '-p', 'breakpad-out'])
-      Command(context, cwd='breakpad-out',
-              cmd=['bash', '../../breakpad/configure',
-                   'CXXFLAGS=-I../..'])  # For third_party/lss
+
+      # Breakpad requires C++11, so use clang and the sysroot rather than
+      # hoping that the host toolchain will provide support.
+      configure_args = []
+      if context.Linux():
+        cc = 'CC=../../third_party/llvm-build/Release+Asserts/bin/clang'
+        cxx = 'CXX=../../third_party/llvm-build/Release+Asserts/bin/clang++'
+        flags = ''
+        if context['arch'] == '32':
+          flags += ' -m32'
+          sysroot_arch = 'i386'
+        else:
+          flags += ' -m64'
+          sysroot_arch = 'amd64'
+        flags += (' --sysroot=../../build/linux/debian_sid_%s-sysroot' %
+                  sysroot_arch)
+        configure_args += [cc + flags, cxx + flags]
+        configure_args += ['CXXFLAGS=-I../..',  # For third_party/lss
+                           'LDFLAGS=-fuse-ld=lld']
+      try:
+        Command(
+            context,
+            cwd='breakpad-out',
+            cmd=['bash', '../../breakpad/configure'] + configure_args)
+      except:
+        f = open(os.path.join('breakpad-out', 'config.log')).read()
+        print(f)
+        raise
+
     with Step('breakpad make', status):
       Command(context, cmd=['make', '-j%d' % context['max_jobs'],
                             # This avoids a broken dependency on
@@ -449,29 +472,8 @@ def BuildScript(status, context):
   ### END tests ###
 
   ### BEGIN GN tests ###
-  # Non-Linux can build non-x86 untrusted code, but can't build or run
-  # the trusted code and so cannot test.
-  if can_use_gn and (context.Linux() or context['arch'] in ('32', '64')):
-    gn_sel_ldr = os.path.join(gn_out_trusted, 'sel_ldr')
-    if context.Windows():
-      gn_sel_ldr += '.exe'
-    gn_extra = [
-        'force_sel_ldr=' + gn_sel_ldr,
-        'force_irt=' + os.path.join(gn_out_irt, 'irt_core.nexe'),
-        'perf_prefix=gn_',
-        ]
-    if context.Linux():
-      gn_extra.append('force_bootstrap=' +
-                      os.path.join(gn_out_trusted, 'nacl_helper_bootstrap'))
-    def RunGNTests(step_suffix, extra_scons_modes, suite_suffix):
-      for suite in ['small_tests', 'medium_tests', 'large_tests']:
-        with Step(suite + step_suffix + ' (GN)', status, halt_on_fail=False):
-          SCons(context,
-                mode=context['default_scons_mode'] + extra_scons_modes,
-                args=[suite + suite_suffix] + gn_extra)
-    if not context['use_glibc']:
-      RunGNTests('', [], '')
-    RunGNTests(' under IRT', ['nacl_irt_test'], '_irt')
+  DoGNTest(status, context, using_gn, 'gn_', ' (GN)')
+  DoGNTest(status, context, using_gn_clang, 'gn_clang_', '(GN, Clang)')
   ### END GN tests ###
 
 
@@ -484,12 +486,11 @@ def Main():
   if context.Windows():
     SetupWindowsEnvironment(context)
   elif context.Linux():
-    if context['android']:
-      SetupAndroidEnvironment(context)
-    else:
+    if not context['android']:
       SetupLinuxEnvironment(context)
   elif context.Mac():
-    SetupMacEnvironment(context)
+    # No setup to do for Mac.
+    pass
   else:
     raise Exception("Unsupported platform.")
   RunBuild(BuildScript, status)
@@ -501,7 +502,7 @@ def TimedMain():
     Main()
   finally:
     time_taken = time.time() - start_time
-    print 'RESULT BuildbotTime: total= %.3f minutes' % (time_taken / 60)
+    print('RESULT BuildbotTime: total= %.3f minutes' % (time_taken / 60))
 
 
 if __name__ == '__main__':
