@@ -205,7 +205,7 @@ TranslatorPatterns = [
   ( '-fPIC',           "env.set('PIC', '1')\n"
                        "env.set('SZ_UNSUPPORTED', '1')"),
 
-  ( '(--build-id)',    "env.append('LD_FLAGS', $0)"),
+  ( '(--build-id(?:=.+)?)',    "env.append('LD_FLAGS', $0)"),
   ( '-(split-module|threads)=([0-9]+|auto|seq)', "env.set('SPLIT_MODULE', $1)"),
   ( '-split-module-sched=(.*)', "env.set('SPLIT_MODULE_SCHED', $0)"),
   ( '-stream-bitcode', "env.set('STREAM_BITCODE', '1')"),
@@ -279,15 +279,31 @@ def SetUpArch():
   # currently offers no optimized path for reading the thread pointer.
   if env.getone('TARGET_OS') != 'nacl' or env.getbool('NONSFI_NACL'):
     env.append('LLC_FLAGS_ARCH', '-mtls-use-call')
-  # For Subzero, determine -target and -sandbox options.
-  env.append('SZ_FLAGS_ARCH', '--sandbox=' +
-             ('1' if env.getone('TARGET_OS') == 'nacl' else '0'))
-  env.append('SZ_FLAGS_ARCH', '--target=' + base_arch.lower())
-  if base_arch != 'X8632':
+  # For Subzero, determine -target, -sandbox, and -nonsfi options.
+  is_sandbox = '0'
+  is_nonsfi = '0'
+  if env.getone('TARGET_OS') == 'nacl':
+    if env.getbool('NONSFI_NACL'):
+      is_nonsfi = '1'
+    else:
+      is_sandbox = '1'
+
+  # Subzero arch setup below.
+  if base_arch not in ('X8632', 'X8664', 'ARM'):
     env.set('SZ_UNSUPPORTED', '1')
     # Hard-fail on an unsupported architecture.
     if env.getbool('USE_SZ'):
       Log.Fatal('Unsupported architecture when using --sz: ' + base_arch)
+    return
+
+  sz_target = {
+      'arm': 'arm32',
+      'x8632': 'x8632',
+      'x8664': 'x8664',
+  }[base_arch.lower()]
+  env.append('SZ_FLAGS_ARCH', '--sandbox=' + is_sandbox)
+  env.append('SZ_FLAGS_ARCH', '--nonsfi=' + is_nonsfi)
+  env.append('SZ_FLAGS_ARCH', '--target=' + sz_target)
   # This is a fine place to map OPT_LEVEL to the Subzero equivalent, with
   # default of -O2.
   sz_opt_map = {
@@ -488,11 +504,16 @@ def RunHostLD(infile, outfile):
                       infile])
   lib_dir = (env.getone('BASE_LIB_NATIVE')
              + 'x86-32-%s/lib' % env.getone('TARGET_OS'))
+  # TODO(stichnot): Consider making fully linked executable smaller by packaging
+  # the .o files into an archive, and/or building the .o files with
+  # -function-sections and linking with -gc-sections.
   args = ['gcc', '-m32', infile, '-o', outfile,
           os.path.join(lib_dir, 'unsandboxed_irt.o'),
           os.path.join(lib_dir, 'irt_random.o'),
           os.path.join(lib_dir, 'irt_query_list.o'),
-          '-lpthread']
+          os.path.join(lib_dir, 'szrt.o'),
+          os.path.join(lib_dir, 'szrt_ll.o'),
+          '-lm', '-lpthread']
   if env.getone('TARGET_OS') == 'linux':
     args.append('-lrt')  # For clock_gettime()
   driver_tools.Run(args)
