@@ -24,8 +24,6 @@
 #include "native_client/src/shared/platform/nacl_sync_checked.h"
 #include "native_client/src/shared/platform/nacl_time.h"
 
-#include "native_client/src/shared/srpc/nacl_srpc.h"
-
 #include "native_client/src/trusted/desc/nacl_desc_base.h"
 #include "native_client/src/trusted/perf_counter/nacl_perf_counter.h"
 
@@ -41,7 +39,6 @@
 #include "native_client/src/trusted/service_runtime/nacl_text.h"
 #include "native_client/src/trusted/service_runtime/sel_memory.h"
 #include "native_client/src/trusted/service_runtime/sel_ldr.h"
-#include "native_client/src/trusted/service_runtime/sel_ldr_thread_interface.h"
 #include "native_client/src/trusted/service_runtime/sel_util.h"
 #include "native_client/src/trusted/service_runtime/sel_addrspace.h"
 
@@ -613,9 +610,11 @@ int NaClCreateMainThread(struct NaClApp     *nap,
   if (0 != nap->dynamic_text_start) {
     auxv_entries++;
   }
-  ptr_tbl_size = (((NACL_STACK_GETS_ARG ? 1 : 0) +
-                   (3 + argc + 1 + envc + 1 + auxv_entries * 2)) *
-                  sizeof(uint32_t));
+  ptr_tbl_size = 3 + argc + 1 + envc + 1 + auxv_entries * 2;
+#if NACL_STACK_GETS_ARG
+  ptr_tbl_size++;
+#endif
+  ptr_tbl_size *= sizeof(uint32_t);
 
   if (SIZE_T_MAX - size < ptr_tbl_size) {
     NaClLog(LOG_WARNING,
@@ -656,10 +655,12 @@ int NaClCreateMainThread(struct NaClApp     *nap,
    * the main argument block.  For other machines, this is passed
    * in a register and that's set in NaClStartThreadInApp.
    */
-  if (NACL_STACK_GETS_ARG) {
-    uint32_t *argloc = p++;
+#if NACL_STACK_GETS_ARG
+  {
+    uint32_t *argloc = p++;  /* Prevent unsequenced access to p in next line. */
     *argloc = (uint32_t) NaClSysToUser(nap, (uintptr_t) p);
   }
+#endif
 
   *p++ = 0;  /* Cleanup function pointer, always NULL.  */
   *p++ = envc;
